{
open Parser
}


let space = [' ' '\t' '\r' '\n']
let lower = ['a' - 'z']
let upper = ['A' - 'Z']
let digit = ['0' - '9']
let alpha = (lower | upper | '_')
let ident = (alpha | digit | '\'' | '-')

rule token = parse
  | space+           { token lexbuf }
  | "/*"             { comment lexbuf; token lexbuf } 
  | '('              { LPAREN }
  | ')'              { RPAREN }
  | '{'              { LCURRY }
  | '}'              { RCURRY }
  | '['              { LBRACKET }
  | ']'              { RBRACKET }
  | '|'              { VERTICAL }
  | '*'              { STAR }
  | '?'              { QUESTION }      
  | '+'              { PLUS }
  | '='              { EQUAL }
  | ','              { COMMA }
  | "<->"            { LRARROW }
  | "@..."           { ANYATTR }
  | "AnyAttr"        { ANYATTR }
  | "relation"       { RELATION }
  | "where"          { WHERE }
  | "var"            { VAR }
  | "as"             { AS }
  | "AnyElem"        { ANYELEM }
  | "String"         { ANYSTRING }
  | '"'              { STRING(immstr lexbuf) }
  | alpha ident*     { IDENT(Lexing.lexeme lexbuf) }
  | '@' alpha ident* { let x = Lexing.lexeme lexbuf in 
                       let len = String.length x in
                       ATTR(String.sub x 1 (len - 1)) }
  | eof              { EOF }
  | _                { failwith ("unknown token detected : " ^ (Lexing.lexeme lexbuf)) }

and comment = parse
  | "*/"             { () }
  | "/*"             { comment lexbuf; comment lexbuf }
  | eof              { failwith "comment is not ended." }
  | _                { comment lexbuf }

and immstr = parse
  | "\\n"            { "\n" ^ (immstr lexbuf) }
  | "\\r"            { "\r" ^ (immstr lexbuf) }
  (* | '\\' _           { let s = (Laxing.make 1 (Lexing.lexeme_char lexbuf 1)) in s ^ (immstr lexbuf) }  *)
  | '"'              { "" }
  | eof              { failwith "unexpected \"" }
  | [^'\\' '"']+     { let s = (Lexing.lexeme lexbuf) in s ^ (immstr lexbuf) } 
  | _                { let s = (Lexing.lexeme lexbuf) in s ^ (immstr lexbuf) } (* Insurance *)





