(**
 @file    main.ml
 @brief   Main
 @author  shinyak
 @version $Rev: 270 $

 Shinya KAWANAKA (C) 2004 All Rights Reserved.
 $Id: main.ml 270 2005-05-08 10:25:02Z shinyak $
*)

open Util
(* module StringMap = Map.Make(String) *)

(** brief show usage *)
let show_usage () =
  print_endline "bitrans -r(-l) <relation-file> <xml-file>"
;;


let main is_reverse relation_file xml_file =

  (* prdbg_endline "Pass1: read XML..."; *)
  let input_xml = ReadXML.read_xml xml_file in
  (* ReadXML.prdbg_xmltree input_xml; *)
  (* prerr_endline "Pass1: OK."; *)

  (* prdbg_endline "Pass2: read relation..."; *)
  let relations : Syntax.relation list = Reader.relations_from_file relation_file in
  let relations = ProcessAttr.process_attr relations in
  (* List.iter Syntax.prdbg_relation relations; *)
  (* prerr_endline "Pass2: OK"; *)

  (* prdbg_endline "Pass3: Pre Process"; *)
  let relations = PreProcess.process is_reverse relations in
  (* List.iter Syntax.prdbg_relation relations; *)
  Syntax.check_nullable relations;
  (* prerr_endline "Pass3: OK"; *)


  (* prdbg_endline "Pass4: create automata..."; *)
  let (lhs_am, rhs_am) = MarkingAutomata.marking_automata_of_relations "top" relations in
  (* MarkingAutomata.prdbg_marking_automata lhs_am;
  MarkingAutomata.prdbg_marking_automata rhs_am; *)
  (* StringMap.iter (fun k v -> prdbg_endline k) lhs_am;
  StringMap.iter (fun k v -> prdbg_endline k) rhs_am; *)
  (* prerr_endline "Pass4: OK.";
  prerr_endline "creating automata OK"; *)


  (* prdbg_endline "Pass5: create binding..."; *)
  let input_bindings = Binding.create_binding lhs_am input_xml in
  let converted_bindings = List.map (fun x -> 
    Convert.inputbinding_of_result "top" relations x
  ) input_bindings in
  (* prerr_endline "Pass5: OK"; *)


  (* prdbg_endline "Pass6: output_binding";
  List.iter (fun x ->
    prdbg_endline "{";
    Binding.Result.print x;
    prdbg_endline "}";
  ) input_bindings; *)
  (* prerr_endline "Pass6: OK"; *)

  (* prerr_string "input_binding OK.  ";
  prerr_int (List.length converted_bindings);
  prerr_newline (); *)

  (* prdbg_endline "Pass7: output XML..."; *)
  let output_xml : Syntax.xml_tree = Output.output_xml rhs_am converted_bindings in
  (* prerr_endline "Pass7: OK"; *)
  
  (* prdbg_endline "------------------------------";
     ReadXML.prdbg_xmltree output_xml;
     prdbg_newline (); *)

  (* prdbg_endline "Pass8: binary tree to XML..."; *)
  ReadXML.output_xml stdout output_xml;
  (* prerr_endline "Pass8: OK"; *)

  (* prerr_endline "ALL OK!" *)
;;


(** *)
let _ =
  if Array.length Sys.argv < 4 then begin
    show_usage ();
    exit 0
  end;

  let is_reverse =
    match Sys.argv.(1) with
        "-r" -> true
      | "-l" -> false
      | _ -> 
	  show_usage (); 
	  exit 0
  in

  let relation_file = Sys.argv.(2) in
  let xml_file = Sys.argv.(3) in
  main is_reverse relation_file xml_file
;;

