%{
(**
 @file    parser.mly
 @brief   Implementation of automata
 @author  shinyak
 @version $Rev: 268 $

 Shinya KAWANAKA (C) 2004 All Rights Reserved.
 $Id: parser.mly 268 2005-04-24 11:38:59Z shinyak $
*)

%}

%token EOF
%token RELATION
%token LRARROW
%token WHERE
%token LPAREN
%token RPAREN
%token LCURRY
%token RCURRY
%token LBRACKET
%token RBRACKET
%token VERTICAL
%token EQUAL
%token COMMA
%token STAR
%token QUESTION
%token PLUS
%token VAR
%token AS
%token ANYELEM
%token ANYSTRING
%token<string> STRING
%token ANYATTR


%token<string> IDENT
%token<string> ATTR


%left COMMA
%left VERTICAL
%nonassoc BINDING
%left QUESTION
%right STAR, PLUS


%type <Syntax.relation list> exps
%start exps

%type <Syntax.pattern> pattern
%start pattern

%%

exps:
  | exp exps { $1 :: $2 }
  | exp      { [$1] }

exp:
  | RELATION ident EQUAL relation where_constraint {
      let (lhs, rhs) = $4 in
      { Syntax.name  = $2;
        Syntax.lhs   = lhs;
	Syntax.rhs   = rhs;
	Syntax.where = $5;
      }
    }

relation:
    pattern LRARROW pattern { ($1, $3) }

pattern:
  | /* epsilon */                       { Syntax.PEmptyElem }
  | LPAREN pattern RPAREN               { $2 }
  | ANYELEM                             { Syntax.PAnyElem }
  | ANYSTRING                           { Syntax.PAnyString }
  | STRING                              { Syntax.PString ($1) }
  | ANYATTR                             { Syntax.PAnyAttr }
  | ATTR LBRACKET pattern RBRACKET { Syntax.PAttr ($1, $3) }
  | ident LBRACKET pattern RBRACKET     { Syntax.PLabel ($1, $3) }
  | pattern COMMA pattern               { Syntax.PSeq ($1, $3) }
  | pattern VERTICAL pattern            { Syntax.POr ($1, $3) }
  | pattern STAR                        { Syntax.PStar ($1) }
  | pattern PLUS                        { Syntax.PSeq ($1, Syntax.PStar($1)) }
  | pattern QUESTION                    { Syntax.POr ($1, Syntax.PEmptyElem) }
  | VAR ident AS pattern %prec BINDING  { Syntax.PVar ($2, $4) }
  | VAR ident                           { Syntax.PVarAny ($2) }

where_constraint:
  | /* epsilon */ { [] }
  | WHERE where_clauses { $2 }

where_clauses:
  | where_clause                     { [$1] }
  | where_clause COMMA where_clauses { $1 :: $3 }

where_clause:
  | ident LPAREN ident COMMA ident RPAREN { ($1, $3, $5) }


ident:
  | IDENT { $1 }

  


