(**
 @file    preProcess.ml
 @brief   Implementation of pre process
 @author  shinyak
 @version $Rev$

 Shinya KAWANAKA (C) 2004 All Rights Reserved.
 $Id$
*)

open Util
open Syntax

let star_process pattern = 
  let star_pattern pattern = 
    if varset_of_pattern pattern = StringSet.empty then
      PEmptyElem
    else
      PStar pattern
  in
  let rec iter (is_in_starchild : bool) (pattern : pattern) : pattern =
    match pattern with
      | PEmptyElem         ->
          if is_in_starchild then star_pattern pattern else pattern
      | PAnyElem           ->
          if is_in_starchild then star_pattern pattern else pattern
      | PAnyString         ->
          if is_in_starchild then star_pattern pattern else pattern
      | PString _          ->
          if is_in_starchild then star_pattern pattern else pattern          
      | PAnyAttr           ->
          if is_in_starchild then star_pattern pattern else pattern                    
      | PAttr (ident, pat) ->
          let pat' = iter false pat in
          if is_in_starchild then star_pattern (PAttr (ident, pat')) else PAttr (ident, pat')
      | PAttrLs _          ->
          if is_in_starchild then star_pattern pattern else pattern
      | PAttrGr _          ->
          if is_in_starchild then star_pattern pattern else pattern          
      | PAttrLsGr _        ->
          if is_in_starchild then star_pattern pattern else pattern          
      | PLabel (lbl, pat)  ->
          let pat' = iter false pat in
          if is_in_starchild then star_pattern (PLabel (lbl, pat')) else PLabel (lbl, pat')
      | PStar pat          ->
          let pat' = iter true pat in
          if is_in_starchild then (PStar pat') else pat'
      | PSeq (pat1, pat2)  ->
          let pat1' = iter false pat1
          and pat2' = iter false pat2 in
          if is_in_starchild then star_pattern (PSeq (pat1', pat2')) else PSeq (pat1', pat2')
      | POr (pat1, pat2)   ->
          let pat1' = iter is_in_starchild pat1
          and pat2' = iter is_in_starchild pat2 in
          if is_in_starchild then (PSeq (pat1', pat2')) else (POr (pat1', pat2'))
      | PVar (ident, pat)  ->
          (* don't touch pat. *)
          if is_in_starchild then PStar pattern else pattern
      | PVarAny ident      ->
          if is_in_starchild then PStar pattern else pattern          
  in
  iter false pattern
;;



let process (is_reverse : bool) (relations : relation list) = 
  List.map (fun rel ->
    if is_reverse then begin
      { name    = rel.name;
        lhs     = rel.rhs;
        rhs     = star_process rel.lhs;
        where   = List.map (fun (name, lhs, rhs) -> (name, rhs, lhs)) rel.where;
      }
    end else begin
      { name    = rel.name;
        lhs     = rel.lhs;
        rhs     = star_process rel.rhs;
        where   = rel.where;
      }
    end
  ) relations
;;

