(**
 @file    processAttr.ml
 @brief   Implementation of automata
 @author  shinyak
 @version $Rev$

 Shinya KAWANAKA (C) 2004 All Rights Reserved.
 $Id$
*)

open Util
open Syntax

let compose_attr_elem (lst : (pattern list * pattern) list) : pattern =

  let compare_attr x y =
    match (x, y) with
      | (PAttr (ident1, pat1), PAttr (ident2, pat2)) -> compare ident1 ident2
      | _ -> failwith "shouldn't happen"
  in

  let process_attributes attributes elem : pattern =
    (* first, check attributes has @... or not. *)
    if List.mem PAnyAttr attributes then begin
      let attributes = List.filter (fun x -> x <> PAnyAttr) attributes in
      let attributes = List.sort compare_attr attributes in
      let rec iter = function
        | [] -> failwith "shouldn't happen"
        | (PAttr (ident, pat)) :: [] -> 
            PSeq (PAttr (ident, pat), PStar (PAttrGr ident))
        | (PAttr (ident1, pat1)) :: (PAttr (ident2, pat2)) :: rest ->
            PSeq (PAttr (ident1, pat1), PSeq (PStar (PAttrLsGr (ident1, ident2)), iter ((PAttr (ident2, pat2)) :: rest)))
        | _ -> failwith "shouldn't happen"
      in

      match attributes with
        | [] ->
            PSeq (PStar (PAnyAttr), elem)
        | PAttr (ident, pat) :: xs->
            PSeq (PStar (PAttrLs ident), PSeq (iter attributes, elem))
        | _ ->
            failwith "shouldn't happen"

    end else begin
      let attributes = List.sort compare_attr attributes in
      List.fold_right (fun x y ->
        PSeq (x, y)
      ) attributes elem
    end
  in

  let new_lst : pattern list = List.map (fun (attr, elem) ->
    if attr = [] then
      elem
    else
      process_attributes attr elem
  ) lst in

  (* combine with 'or' *)
  List.fold_left (fun x y ->
    POr (x, y)
  ) (List.hd new_lst) (List.tl new_lst) 
;;


let process_attr_aux (pattern : pattern) =
  (* return (attrs, elems). list means 'or'. *)
  let rec iter (pattern : pattern) : (pattern list * pattern) list = match pattern with 
    | PEmptyElem -> [([], PEmptyElem)]
    | PAnyElem   -> [([], PAnyElem)]
    | PAnyString -> [([], PAnyString)]
    | PString str  -> [([], PString str)]
    | PAnyAttr   -> [([PAnyAttr], PEmptyElem)]
    | PAttr (elem, child) -> 
        let pat = iter child in
        [([PAttr (elem, compose_attr_elem pat)], PEmptyElem)]
    | PAttrLs _ 
    | PAttrGr _
    | PAttrLsGr _ -> failwith "shouldn't happen"
    | PLabel (elem, child) ->
        let pat = iter child in
        [([], PLabel (elem, compose_attr_elem pat))]
    | PStar child ->
        let pat = iter child in
        [([], PStar (compose_attr_elem pat))]
    | PSeq (lhs, rhs) ->
        let lhs' = iter lhs
        and rhs' = iter rhs in
        List.flatten (List.map (fun (attr_lhs, elem_lhs) -> 
          List.map (fun (attr_rhs, elem_rhs) ->
            let attr = attr_lhs @ attr_rhs in
            let elem =
              if elem_lhs = PEmptyElem then elem_rhs
              else if elem_rhs = PEmptyElem then elem_lhs
              else PSeq (elem_lhs, elem_rhs)
            in
            (attr, elem)
          ) rhs'
        ) lhs')
    | POr (lhs, rhs) -> 
        let lhs' = iter lhs
        and rhs' = iter rhs in
        lhs' @ rhs'
    | PVar (ident, pattern) ->
        [([], PVar (ident, pattern))]
    | PVarAny ident ->
        [([], PVarAny ident)]
  in

  compose_attr_elem (iter pattern)
;;

(**
   @brief  
*)
let process_attr (relations : relation list) =
  List.map (fun relation ->
    { name = relation.name;
      lhs  = process_attr_aux relation.lhs;
      rhs  = process_attr_aux relation.rhs;
      where = relation.where;
    }
  ) relations
;;
