(**
 @file    readXML
 @brief   Implementation of automata
 @author  shinyak
 @version $Rev: 268 $

 Shinya KAWANAKA (C) 2004 All Rights Reserved.
 $Id: readXML.ml 268 2005-04-24 11:38:59Z shinyak $
*)

open Pxp_document
open Pxp_types
open Pxp_yacc
open Util
open Syntax

type docnode = ('a Pxp_document.node Pxp_document.extension as 'a) Pxp_document.node

(*
let rec prdbg_xmltree = function
  | XMLLeaf -> prdbg_string " Leaf "
  | XMLString (str, t1, next) -> 
      prdbg_string (" Str (" ^ str ^ ")");
      prdbg_xmltree next
  | XMLAttr (l, t1, t2) ->
      prdbg_string "@";
      prdbg_string l;
      prdbg_string "=[";
      prdbg_xmltree t1;
      prdbg_string "] ";
      prdbg_xmltree t2
  | XMLNode (l, t1, t2) -> 
      prdbg_string l;
      prdbg_string "(";
      prdbg_xmltree t1;
      prdbg_xmltree t2;
      prdbg_string ")";      
;;
*)

let rec print_xmltree = function
  | XMLLeaf -> print_string " Leaf "
  | XMLString (str, t1, next) -> 
      print_string (" Str (" ^ str ^ ")");
      print_xmltree next
  | XMLAttr (l, t1, t2) ->
      print_string "@";
      print_string l;
      print_string "=[";
      print_xmltree t1;
      print_string "] ";
      print_xmltree t2
  | XMLNode (l, t1, t2) -> 
      print_string l;
      print_string "(";
      print_xmltree t1;
      print_xmltree t2;
      print_string ")";      
;;

let is_all_space string =
  let rec iter s i n =
    if i = n then true
    else if s.[i] = ' ' || s.[i] = '\n' || s.[i] = '\r' || s.[i] = '\t' then 
      iter s (succ i) n
    else false
  in
  iter string 0 (String.length string)
;;

let rec create_binary_tree (nd : docnode) tail =
  let nodetype = nd # node_type in
  match nodetype with
    | T_element name ->
	(* prdbg_endline ("name: " ^ name); *)
	let subnodes = nd # sub_nodes in
        let attribute_names : string list = nd # attribute_names in
        let attributes : (string * string) list =
          List.map (fun (attr_name : string) ->
            match (nd # attribute attr_name) with
              | Value z -> (attr_name, z)
              | _ -> failwith "not support"
          ) attribute_names 
        in
        let attributes = List.sort Pervasives.compare attributes in

	let left = match subnodes with
	  | [] ->
	      (* prdbg_endline "subnodes is empty."; *)
	      XMLLeaf
	  | s :: ss ->
	      (* prdbg_endline ("subnodes num is " ^ (string_of_int (List.length subnodes))); *)
	      create_binary_tree s ss 
	in
	let right = match tail with
	    [] -> XMLLeaf
	  | s :: ss -> create_binary_tree s ss
	in

        let left_attred = List.fold_right (fun (name, value) b ->
          XMLAttr (name, XMLString (value, XMLLeaf, XMLLeaf), b)
        ) attributes left in

	XMLNode (name, left_attred, right)
    | T_data -> begin
	match tail with
	  | [] ->
	      if is_all_space (nd # data) then begin
		(* prdbg_endline "leaf"; *)
		XMLLeaf		
	      end else begin
                (* prdbg_endline "string"; *)
                XMLString ((nd # data), XMLLeaf, XMLLeaf)
	      end
	  | t :: ts ->
              if is_all_space (nd # data) then begin
                (* ignore empty sequence *)
	        create_binary_tree t ts
              end else begin
                XMLString ((nd # data), XMLLeaf, create_binary_tree t ts)
              end
      end
    | _ ->
	assert false
;;


let read_xml xml_file : xml_tree =
  let doc = parse_wfdocument_entity default_config (from_file xml_file) default_spec in
  create_binary_tree (doc # root) []
;;


let output_xml out_chan xmltree : unit =
  let rec iter process_attr xmltree =
    match xmltree with
      | XMLLeaf -> 
          if process_attr then 
            output_string out_chan ">"
      | XMLString (string, child, next) ->
          if process_attr then
            output_string out_chan ">";
          
          output_string out_chan string;
          iter false next;
      | XMLAttr   (attr, child, next) ->
          output_string out_chan " ";
          output_string out_chan attr;
          output_string out_chan "=\"";

          iter false child;
          output_string out_chan "\"";
          iter process_attr next;
      | XMLNode   (lbl, child, next) ->
          if process_attr then
            output_string out_chan ">";
          
          output_string out_chan ("<" ^ lbl);
          iter true child;
          output_string out_chan ("</" ^ lbl ^ ">");
          iter false next
  in
  iter false xmltree;
  output_char out_chan '\n'
;;
